use std::collections::BTreeMap;

use anyhow::Error;
use cozo::{DbInstance, DataValue, ScriptMutability};

use crate::{state::types::PeerId, comm::Peer, run_query};



pub fn add_peer(db: &DbInstance, id: &PeerId, name: &str) -> anyhow::Result<()> {
    let params = vec![
        ("id", DataValue::Str(serde_json::to_string(id)?.into())),
        ("name", DataValue::Str(serde_json::to_string(name)?.into()))
    ];
    
    match run_query!(&db, ":put peers {id => name}", params, ScriptMutability::Mutable) {
        Ok(_) => Ok(()),
        Err(report) => Err(Error::msg(format!("Query failed: {}", report))),
    }
}

pub fn get_peers(db: &DbInstance) -> anyhow::Result<Vec<Peer>> {
    let result = db.run_script("
    ?[id, name] := *peers{id, name}
    ", Default::default(), cozo::ScriptMutability::Immutable);
    match result {
        Ok(rows) => {
            Ok(
                rows.rows.into_iter().map(
                    |row| {
                        match row.as_slice() {
                            [DataValue::Str(id_string), DataValue::Str(name_string)] => {
                                if let Ok(id) = serde_json::from_str(&id_string) {
                                    Some(Peer::new(id, name_string.as_str().to_string()))
                                }
                                else {
                                    None
                                }},
                            _ => None
                        }
                    }
                )
                .flatten()
                .collect()
            )
        },
        Err(report) => Err(Error::msg(format!("Query failed: {}", report))),
    }
}
